@@ -57,27 +57,27 @@ class LabelInference:
 
         debug.println(debug.LEVEL_FINE, "INFER label for: %s" % obj)
         if not obj:
-            return None
+            return None, []
 
         if focusedOnly and not obj.getState().contains(pyatspi.STATE_FOCUSED):
             debug.println(debug.LEVEL_FINE, "INFER - object not focused")
-            return None
+            return None, []
 
-        result = None
+        result, objects = None, []
         if not result:
-            result = self.inferFromTextLeft(obj)
+            result, objects = self.inferFromTextLeft(obj)
             debug.println(debug.LEVEL_FINE, "INFER - Text Left: %s" % result)
         if not result or self._preferRight(obj):
-            result = self.inferFromTextRight(obj) or result
+            result, objects = self.inferFromTextRight(obj) or result
             debug.println(debug.LEVEL_FINE, "INFER - Text Right: %s" % result)
         if not result:
-            result = self.inferFromTable(obj)
+            result, objects = self.inferFromTable(obj)
             debug.println(debug.LEVEL_FINE, "INFER - Table: %s" % result)
         if not result:
-            result = self.inferFromTextAbove(obj)
+            result, objects = self.inferFromTextAbove(obj)
             debug.println(debug.LEVEL_FINE, "INFER - Text Above: %s" % result)
         if not result:
-            result = self.inferFromTextBelow(obj)
+            result, objects = self.inferFromTextBelow(obj)
             debug.println(debug.LEVEL_FINE, "INFER - Text Below: %s" % result)
 
         # TODO - We probably do not wish to "infer" from these. Instead, we
@@ -85,16 +85,17 @@ class LabelInference:
         # (i.e. the label is something on screen. Widget name and description
         # are each something other than a label.)
         if not result:
-            result = obj.name
+            result, objects = obj.name, []
             debug.println(debug.LEVEL_FINE, "INFER - Name: %s" % result)
         if not result:
-            result = obj.description
+            result, objects = obj.description, []
             debug.println(debug.LEVEL_FINE, "INFER - Description: %s" % result)
         if result:
             result = result.strip()
+            result = result.replace("\n", " ")
 
         self.clearCache()
-        return result
+        return result, objects
 
     def clearCache(self):
         """Dumps whatever we've stored for performance purposes."""
@@ -134,11 +135,10 @@ class LabelInference:
         """Returns True if we should not permit inference based on text below
         the object obj."""
 
-        roles = [pyatspi.ROLE_COMBO_BOX,
-                 pyatspi.ROLE_LIST,
-                 pyatspi.ROLE_LIST_BOX]
+        roles = [pyatspi.ROLE_ENTRY,
+                 pyatspi.ROLE_PASSWORD_TEXT]
 
-        return obj.getRole() in roles
+        return obj.getRole() not in roles
 
     def _isSimpleObject(self, obj):
         """Returns True if the given object has 'simple' contents, such as text
@@ -168,6 +168,18 @@ class LabelInference:
 
         return True
 
+    def _cannotLabel(self, obj):
+        """Returns True if the given object should not be treated as a label."""
+
+        if not obj:
+            return True
+
+        nonLabelTextRoles = [pyatspi.ROLE_HEADING]
+        if obj.getRole() in nonLabelTextRoles:
+            return True
+
+        return self._isWidget(obj)
+
     def _isWidget(self, obj):
         """Returns True if the given object is a widget."""
 
@@ -183,18 +195,13 @@ class LabelInference:
                        pyatspi.ROLE_TOGGLE_BUTTON,
                        pyatspi.ROLE_COMBO_BOX,
                        pyatspi.ROLE_LIST,
+                       pyatspi.ROLE_LIST_BOX,
                        pyatspi.ROLE_MENU,
                        pyatspi.ROLE_MENU_ITEM,
                        pyatspi.ROLE_ENTRY,
                        pyatspi.ROLE_PASSWORD_TEXT,
                        pyatspi.ROLE_PUSH_BUTTON]
 
-        # Put new-to-pyatspi roles here.
-        try:
-            widgetRoles.append(pyatspi.ROLE_LIST_BOX)
-        except:
-            pass
-
         isWidget = obj.getRole() in widgetRoles
         self._isWidgetCache[hash(obj)] = isWidget
         return isWidget
@@ -236,7 +243,7 @@ class LabelInference:
         if not self._isSimpleObject(obj):
             return ''
 
-        if self._isWidget(obj):
+        if self._cannotLabel(obj):
             return ''
 
         contents = self._script.utilities.getObjectsFromEOCs(obj)
@@ -261,24 +268,15 @@ class LabelInference:
             start, end = self._script.utilities.getHyperlinkRange(obj)
             obj = obj.parent
 
-        boundary = pyatspi.TEXT_BOUNDARY_LINE_START
-
         try:
             text = obj.queryText()
-        except NotImplementedError:
-            pass
-        except AttributeError:
-            debug.println(debug.LEVEL_FINE, "INFER _getLineContents: no obj")
+        except:
+            start = 0
         else:
             if start == None:
                 start = max(0, text.caretOffset)
-            else:
-                start = max(0, start - 1)
-            string, start, end = text.getTextAtOffset(start, boundary)
-            if string.endswith('\n'):
-                start = end
 
-        rv = self._script.utilities.getObjectsFromEOCs(obj, start, boundary)
+        rv = self._script.utilities.getLineContentsAtOffset(obj, start)
         self._lineCache[key] = rv
 
         return rv
@@ -317,26 +315,26 @@ class LabelInference:
             index = len(contents)
 
         onLeft = contents[0:index]
+        start = 0
         for i in range(len(onLeft) - 1, -1, -1):
-            if self._isWidget(onLeft[i][0]):
-                onLeft = onLeft[(i+1):]
+            if self._cannotLabel(onLeft[i][0]):
+                start = i + 1
                 break
 
+        onLeft = onLeft[start:]
         if not (onLeft and onLeft[0]):
-            return None
+            return None, []
 
         lObj, start, end, string = onLeft[-1]
-        string = (string or lObj.name).strip()
-        if not string:
-            return None
-
         lExtents = self._getExtents(lObj, start, end)
         distance = extents[0] - (lExtents[0] + lExtents[2])
         if 0 <= distance <= proximity:
             strings = [content[3] or content[0].name for content in onLeft]
-            return ''.join(strings)
+            result = ''.join(strings)
+            if result.strip():
+                return result, [content[0] for content in onLeft]
 
-        return None
+        return None, []
 
     def inferFromTextRight(self, obj, proximity=25):
         """Attempt to infer the functional/displayed label of obj by
@@ -351,7 +349,7 @@ class LabelInference:
         """
 
         if self._preventRight(obj):
-            return None
+            return None, []
 
         extents = self._getExtents(obj)
         contents = self._getLineContents(obj)
@@ -362,28 +360,28 @@ class LabelInference:
             index = len(contents)
 
         onRight = contents[min(len(contents), index+1):]
+        end = len(onRight)
         for i, item in enumerate(onRight):
-            if self._isWidget(item[0]):
+            if self._cannotLabel(item[0]):
                 if not self._preferRight(obj):
-                    return None
-                onRight = onRight[0:i]
+                    return None, []
+                end = i + 1
                 break
 
+        onRight = onRight[0:end]
         if not (onRight and onRight[0]):
-            return None
+            return None, []
 
         rObj, start, end, string = onRight[0]
-        string = (string or rObj.name).strip()
-        if not string:
-            return None
-
         rExtents = self._getExtents(rObj, start, end)
         distance = rExtents[0] - (extents[0] + extents[2])
         if distance <= proximity or self._preferRight(obj):
             strings = [content[3] or content[0].name for content in onRight]
-            return ''.join(strings)
+            result = ''.join(strings)
+            if result.strip():
+                return result, [content[0] for content in onRight]
 
-        return None
+        return None, []
 
     def inferFromTextAbove(self, obj, proximity=20):
         """Attempt to infer the functional/displayed label of obj by
@@ -399,7 +397,7 @@ class LabelInference:
 
         thisLine = self._getLineContents(obj)
         if not (thisLine and thisLine[0]):
-            return None
+            return None, []
 
         prevObj, start, end, string = thisLine[0]
         if obj == prevObj:
@@ -409,39 +407,42 @@ class LabelInference:
         try:
             text = prevObj.queryText()
         except (AttributeError, NotImplementedError):
-            return None
+            return None, []
 
         objX, objY, objWidth, objHeight = self._getExtents(obj)
         if not (objWidth and objHeight):
-            return None
+            return None, []
 
-        boundary = pyatspi.TEXT_BOUNDARY_LINE_START
-        line = text.getTextAtOffset(start - 1, boundary)
-        string = line[0].strip()
-        if string:
+        start = max(start - 1, 0)
+        prevLine = self._script.utilities.getLineContentsAtOffset(prevObj, start)
+        if not (prevLine and prevLine[0]):
+            return None, []
+
+        prevObj, start, end, string = prevLine[0]
+        if string.strip() and not self._cannotLabel(prevObj):
             x, y, width, height = self._getExtents(prevObj, start, end)
             distance = objY - (y + height)
-            if distance <= proximity:
-                return string
+            if 0 <= distance <= proximity:
+                return string, [prevObj]
 
         while prevObj:
             prevObj = self._getPreviousObject(prevObj)
             x, y, width, height = self._getExtents(prevObj)
             distance = objY - (y + height)
             if distance > proximity:
-                return None
+                return None, []
             if prevObj.getRole() == pyatspi.ROLE_TABLE_CELL \
                and not prevObj in [obj.parent, obj.parent.parent]:
-                return None
+                return None, []
             if distance < 0:
                 continue
             if x + 150 < objX:
                 continue
             string = self._createLabelFromContents(prevObj)
             if string:
-                return string
+                return string, [prevObj]
 
-        return None
+        return None, []
 
     def inferFromTextBelow(self, obj, proximity=20):
         """Attempt to infer the functional/displayed label of obj by
@@ -456,11 +457,11 @@ class LabelInference:
         """
 
         if self._preventBelow(obj):
-            return None
+            return None, []
 
         thisLine = self._getLineContents(obj)
         if not (thisLine and thisLine[0]):
-            return None
+            return None, []
 
         lastObj, start, end, string = thisLine[-1]
         if obj == lastObj:
@@ -469,24 +470,22 @@ class LabelInference:
 
         objX, objY, objWidth, objHeight = self._getExtents(obj)
         if not (objWidth and objHeight):
-            return None
+            return None, []
 
-        boundary = pyatspi.TEXT_BOUNDARY_LINE_START
-        nextLine = self._script.utilities.getObjectsFromEOCs(
-            lastObj, end + 1, boundary)
+        nextLine = self._script.utilities.getLineContentsAtOffset(lastObj, end)
         if not (nextLine and nextLine[0]):
-            return None
+            return None, []
 
         nextObj, start, end, string = nextLine[0]
-        if string.strip():
+        if string.strip() and not self._cannotLabel(nextObj):
             x, y, width, height = self._getExtents(nextObj, start, end)
             distance = y - (objY + objHeight)
             if 0 <= distance <= proximity:
-                return string
+                return string, [nextObj]
 
-        return None
+        return None, []
 
-    def inferFromTable(self, obj):
+    def inferFromTable(self, obj, proximityForRight=50):
         """Attempt to infer the functional/displayed label of obj by looking
         at the contents of the surrounding table cells. Note that this approach
         assumes a simple table in which the widget is the sole occupant of its
@@ -501,77 +500,80 @@ class LabelInference:
         pred = lambda x: x.getRole() == pyatspi.ROLE_TABLE_CELL
         cell = pyatspi.utils.findAncestor(obj, pred)
         if not self._isSimpleObject(cell):
-            return None
+            return None, []
 
         if not cell in [obj.parent, obj.parent.parent]:
-            return None
+            return None, []
 
         pred = lambda x: x.getRole() == pyatspi.ROLE_TABLE
         grid = pyatspi.utils.findAncestor(cell, pred)
         if not grid:
-            return None
+            return None, []
 
         try:
             table = grid.queryTable()
         except NotImplementedError:
-            return None
+            return None, []
 
         index = self._script.utilities.cellIndex(cell)
         row = table.getRowAtIndex(index)
         col = table.getColumnAtIndex(index)
+        objX, objY, objWidth, objHeight = self._getExtents(obj)
 
         if col > 0 and not self._preferRight(obj):
             candidate = table.getAccessibleAt(row, col - 1)
             label = self._createLabelFromContents(candidate)
-            if label:
-                return label
+            if label.strip():
+                return label, [candidate]
 
         if col < table.nColumns and not self._preventRight(obj):
             candidate = table.getAccessibleAt(row, col + 1)
-            label = self._createLabelFromContents(candidate)
-            if label:
-                return label
+            x, y, width, height = self._getExtents(candidate)
+            distance = x - (objX + objWidth)
+            if distance <= proximityForRight or self._preferRight(obj):
+                label = self._createLabelFromContents(candidate)
+                if label.strip():
+                    return label, [candidate]
 
         cellAbove = cellBelow = labelAbove = labelBelow = None
         if row > 0:
             cellAbove = table.getAccessibleAt(row - 1, col)
             labelAbove = self._createLabelFromContents(cellAbove)
             if labelAbove and self._preferTop(obj):
-                return labelAbove
+                return labelAbove, [cellAbove]
 
         if row < table.nRows and not self._preventBelow(obj):
             cellBelow = table.getAccessibleAt(row + 1, col)
             labelBelow = self._createLabelFromContents(cellBelow)
 
         if labelAbove and labelBelow:
-            objX, objY, objWidth, objHeight = self._getExtents(obj)
             aboveX, aboveY, aboveWidth, aboveHeight = self._getExtents(cellAbove)
             belowX, belowY, belowWidth, belowHeight = self._getExtents(cellBelow)
             dAbove = objY - (aboveY + aboveHeight)
             dBelow = belowY - (objY + objHeight)
             if dAbove <= dBelow:
-                return labelAbove
-            return labelBelow
+                return labelAbove, [cellAbove]
+            return labelBelow, [cellBelow]
 
         if labelAbove:
-            return labelAbove
+            return labelAbove, [cellAbove]
         if labelBelow:
-            return labelBelow
+            return labelBelow, [cellBelow]
 
         # None of the cells immediately surrounding this cell seem to be serving
         # as a functional label. Therefore, see if this table looks like a grid
         # of widgets with the functional labels in the first row.
         firstRow = [table.getAccessibleAt(0, i) for i in range(table.nColumns)]
         if not firstRow or list(filter(self._isWidget, firstRow)):
-            return None
+            return None, []
 
         cells = [table.getAccessibleAt(i, col) for i in range(1, table.nRows)]
         cells = [x for x in cells if x != None]
         if [x for x in cells if x.childCount and x[0].getRole() != obj.getRole()]:
-            return None
+            return None, []
 
         label = self._createLabelFromContents(firstRow[col])
         if label:
-            return label
+            return label, [firstRow[col]]
 
-        return None
+        return None, []
