@@ -43,7 +43,6 @@ import orca.speechserver as speechserver
 import orca.orca_state as orca_state
 import orca.speech as speech
 
-from . import script_settings
 from .structural_navigation import StructuralNavigation
 from .braille_generator import BrailleGenerator
 from .speech_generator import SpeechGenerator
@@ -71,24 +70,11 @@ class Script(default.Script):
         default.Script.__init__(self, app)
         self._loadingDocumentContent = False
         self._isBrowser = isBrowser
-
+        self._lastCaretContext = None, -1
         self.sayAllOnLoadCheckButton = None
 
-    def getListeners(self):
-        """Sets up the AT-SPI event listeners for this script."""
-
-        listeners = default.Script.getListeners(self)
-
-        listeners["document:reload"]                        = \
-            self.onDocumentReload
-        listeners["document:load-complete"]                 = \
-            self.onDocumentLoadComplete
-        listeners["document:load-stopped"]                  = \
-            self.onDocumentLoadStopped
-        listeners["object:state-changed:busy"]              = \
-            self.onStateChanged
-
-        return listeners
+        if _settingsManager.getSetting('sayAllOnLoad') == None:
+            _settingsManager.setSetting('sayAllOnLoad', True)
 
     def setupInputEventHandlers(self):
         """Defines InputEventHandler fields for this script that can be
@@ -132,28 +118,18 @@ class Script(default.Script):
         label = guilabels.READ_PAGE_UPON_LOAD
         self.sayAllOnLoadCheckButton = \
             Gtk.CheckButton.new_with_mnemonic(label)
-        self.sayAllOnLoadCheckButton.set_active(script_settings.sayAllOnLoad)
+        self.sayAllOnLoadCheckButton.set_active(
+            _settingsManager.getSetting('sayAllOnLoad'))
         grid.attach(self.sayAllOnLoadCheckButton, 0, 0, 1, 1)
 
         grid.show_all()
 
         return grid
 
-    def setAppPreferences(self, prefs):
-        """Write out the application specific preferences lines and set the
-        new values.
+    def getPreferencesFromGUI(self):
+        """Returns a dictionary with the app-specific preferences."""
 
-        Arguments:
-        - prefs: file handle for application preferences.
-        """
-
-        prefs.writelines("\n")
-        prefix = "orca.scripts.toolkits.WebKitGtk.script_settings"
-        prefs.writelines("import %s\n\n" % prefix)
-
-        value = self.sayAllOnLoadCheckButton.get_active()
-        prefs.writelines("%s.sayAllOnLoad = %s\n" % (prefix, value))
-        script_settings.sayAllOnLoad = value
+        return {'sayAllOnLoad': self.sayAllOnLoadCheckButton.get_active()}
 
     def getBrailleGenerator(self):
         """Returns the braille generator for this script."""
@@ -175,8 +151,7 @@ class Script(default.Script):
         """Returns a list of the structural navigation object types
         enabled in this script."""
 
-        enabledTypes = [StructuralNavigation.ANCHOR,
-                        StructuralNavigation.BLOCKQUOTE,
+        enabledTypes = [StructuralNavigation.BLOCKQUOTE,
                         StructuralNavigation.BUTTON,
                         StructuralNavigation.CHECK_BOX,
                         StructuralNavigation.CHUNK,
@@ -211,6 +186,8 @@ class Script(default.Script):
         - event: the Event
         """
 
+        self._lastCaretContext = event.source, event.detail1
+
         lastKey, mods = self.utilities.lastKeyAndModifiers()
         if lastKey in ['Tab', 'ISO_Left_Tab']:
             return
@@ -227,15 +204,17 @@ class Script(default.Script):
         default.Script.onCaretMoved(self, event)
 
     def onDocumentReload(self, event):
-        """Called when the reload button is hit for a web page."""
+        """Callback for document:reload accessibility events."""
 
-        if event.source.getRole() == pyatspi.ROLE_DOCUMENT_FRAME:
+        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
+        if event.source.getRole() in docRoles:
             self._loadingDocumentContent = True
 
     def onDocumentLoadComplete(self, event):
-        """Called when a web page load is completed."""
+        """Callback for document:load-complete accessibility events."""
 
-        if event.source.getRole() != pyatspi.ROLE_DOCUMENT_FRAME:
+        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
+        if not event.source.getRole() in docRoles:
             return
 
         self._loadingDocumentContent = False
@@ -249,25 +228,24 @@ class Script(default.Script):
         orca.setLocusOfFocus(event, obj, False)
 
         self.updateBraille(obj)
-        if script_settings.sayAllOnLoad \
+        if _settingsManager.getSetting('sayAllOnLoad') \
            and _settingsManager.getSetting('enableSpeech'):
             self.sayAll(None)
 
     def onDocumentLoadStopped(self, event):
-        """Called when a web page load is interrupted."""
+        """Callback for document:load-stopped accessibility events."""
 
-        if event.source.getRole() == pyatspi.ROLE_DOCUMENT_FRAME:
+        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
+        if event.source.getRole() in docRoles:
             self._loadingDocumentContent = False
 
-    def onFocus(self, event):
-        """Called whenever an object gets focus.
-
-        Arguments:
-        - event: the Event
-        """
+    def onFocusedChanged(self, event):
+        """Callback for object:state-changed:focused accessibility events."""
 
         obj = event.source
         role = obj.getRole()
+        self._lastCaretContext = obj, -1
+
         if role == pyatspi.ROLE_LIST_ITEM and obj.childCount:
             return
 
@@ -276,7 +254,7 @@ class Script(default.Script):
                        pyatspi.ROLE_LIST_ITEM,
                        pyatspi.ROLE_RADIO_BUTTON]
         if role in widgetRoles:
-            default.Script.onFocus(self, event)
+            default.Script.onFocusedChanged(self, event)
             return
 
         textRoles = [pyatspi.ROLE_HEADING,
@@ -292,47 +270,29 @@ class Script(default.Script):
             if lastKey in self.CARET_NAVIGATION_KEYS:
                 return
 
-        default.Script.onFocus(self, event)
+        default.Script.onFocusedChanged(self, event)
 
-    def onStateChanged(self, event):
-        """Called whenever an object's state changes.
+    def onBusyChanged(self, event):
+        """Callback for object:state-changed:busy accessibility events."""
 
-        Arguments:
-        - event: the Event
-        """
-
-        if not event.type.startswith("object:state-changed:busy"):
-            default.Script.onStateChanged(self, event)
+        obj = event.source
+        try:
+            role = obj.getRole()
+            name = obj.name
+        except:
             return
 
-        if not event.source \
-           or event.source.getRole() != pyatspi.ROLE_DOCUMENT_FRAME \
-           or not self._isBrowser:
+        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
+        if role not in docRoles or not self._isBrowser:
             return
 
         if event.detail1:
             self.presentMessage(messages.PAGE_LOADING_START)
-        elif event.source.name:
-            self.presentMessage(
-                messages.PAGE_LOADING_END_NAMED % event.source.name)
+        elif name:
+            self.presentMessage(messages.PAGE_LOADING_END_NAMED % name)
         else:
             self.presentMessage(messages.PAGE_LOADING_END)
 
-    def onTextSelectionChanged(self, event):
-        """Called when an object's text selection changes.
-
-        Arguments:
-        - event: the Event
-        """
-
-        # The default script's method attempts to handle various and sundry
-        # complications that simply do not apply here.
-        #
-        spokenRange = self.pointOfReference.get("spokenTextRange") or [0, 0]
-        startOffset, endOffset = spokenRange
-
-        self.speakTextSelectionState(event.source, startOffset, endOffset)
-
     def sayCharacter(self, obj):
         """Speak the character at the caret.
 
@@ -348,7 +308,7 @@ class Script(default.Script):
         objects = self.utilities.getObjectsFromEOCs(obj, boundary=boundary)
         for (obj, start, end, string) in objects:
             if string:
-                speech.speakCharacter(string)
+                self.speakCharacter(string)
             else:
                 speech.speak(self.speechGenerator.generateSpeech(obj))
 
@@ -487,6 +447,8 @@ class Script(default.Script):
         """
 
         def implementsText(obj):
+            if obj.getRole() == pyatspi.ROLE_LIST:
+                return False
             return 'Text' in utils.listInterfaces(obj)
 
         child = obj
@@ -603,13 +565,13 @@ class Script(default.Script):
         if obj.getRole() == pyatspi.ROLE_LINK:
             obj = obj.parent
 
-        document = utils.findAncestor(
-            obj, lambda x: x.getRole() == pyatspi.ROLE_DOCUMENT_FRAME)
-        if not document:
+        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
+        document = utils.findAncestor(obj, lambda x: x.getRole() in docRoles)
+        if not document or document.getState().contains(pyatspi.STATE_BUSY):
             return
 
         allTextObjs = utils.findAllDescendants(
-            document, lambda x: 'Text' in utils.listInterfaces(x))
+            document, lambda x: x and 'Text' in utils.listInterfaces(x))
         allTextObjs = allTextObjs[allTextObjs.index(obj):len(allTextObjs)]
         textObjs = [x for x in allTextObjs if x.parent not in allTextObjs]
         if not textObjs:
