@@ -37,6 +37,7 @@ from . import settings
 from . import settings_manager
 from . import sound
 from . import text_attribute_names
+from . import acss
 
 class Pause:
     """A dummy class to indicate we want to insert a pause into an
@@ -222,6 +223,48 @@ class SpeechGenerator(generator.Generator):
             result.extend(acss)
         return result
 
+    def _generateClickable(self, obj, **args):
+        if _settingsManager.getSetting('onlySpeakDisplayedText') \
+           or _settingsManager.getSetting('speechVerbosityLevel') \
+               == settings.VERBOSITY_LEVEL_BRIEF:
+            return []
+
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateClickable(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
+    def _generateHasLongDesc(self, obj, **args):
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateHasLongDesc(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
+    def _generateAvailability(self, obj, **args):
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateAvailability(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
+    def _generateRequired(self, obj, **args):
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateRequired(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
     def _generateTextRole(self, obj, **args):
         """A convenience method to prevent the pyatspi.ROLE_PARAGRAPH role
         from being spoken. In the case of a pyatspi.ROLE_PARAGRAPH
@@ -257,7 +300,9 @@ class SpeechGenerator(generator.Generator):
         acss = self.voice(SYSTEM)
         role = args.get('role', obj.getRole())
 
-        doNotPresent = [pyatspi.ROLE_UNKNOWN]
+        doNotPresent = [pyatspi.ROLE_UNKNOWN,
+                        pyatspi.ROLE_FILLER,
+                        pyatspi.ROLE_EXTENDED]
 
         # egg-list-box, e.g. privacy panel in gnome-control-center
         if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
@@ -917,9 +962,19 @@ class SpeechGenerator(generator.Generator):
         acss = self.voice(DEFAULT)
         result = generator.Generator._generateCurrentLineText(self, obj, **args)
         if result:
+            if result == ["\n"]:
+                result = [messages.BLANK]
             result.extend(acss)
         return result
 
+    def _generateDisplayedText(self, obj, **args):
+        acss = self.voice(DEFAULT)
+        result = generator.Generator._generateDisplayedText(self, obj, **args)
+        if result:
+            result.extend(acss)
+
+        return result
+
     def _getCharacterAttributes(self,
                                 obj,
                                 text,
@@ -944,11 +999,12 @@ class SpeechGenerator(generator.Generator):
 
         if attributesDictionary:
             for key in keys:
-                localizedKey = text_attribute_names.getTextAttributeName(key)
+                localizedKey = text_attribute_names.getTextAttributeName(
+                    key, self._script)
                 if key in attributesDictionary:
                     attribute = attributesDictionary[key]
-                    localizedValue = \
-                        text_attribute_names.getTextAttributeName(attribute)
+                    localizedValue = text_attribute_names.getTextAttributeName(
+                        attribute, self._script)
                     if attribute:
                         # If it's the 'weight' attribute and greater than 400,
                         # just speak it as bold, otherwise speak the weight.
@@ -1115,6 +1171,11 @@ class SpeechGenerator(generator.Generator):
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
             return []
 
+        try:
+            text = obj.queryText()
+        except NotImplementedError:
+            return []
+
         result = []
         acss = self.voice(SYSTEM)
 
@@ -1367,7 +1428,7 @@ class SpeechGenerator(generator.Generator):
     def _generateFocusedItem(self, obj, **args):
         result = []
         role = args.get('role', obj.getRole())
-        if role != pyatspi.ROLE_LIST:
+        if role not in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX]:
             return result
 
         try:
@@ -1393,35 +1454,26 @@ class SpeechGenerator(generator.Generator):
         and the position of the current item. This object will be an icon
         panel or a layered pane.
         """
+
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
             return []
 
+        container = obj
+        if not 'Selection' in pyatspi.listInterfaces(container):
+            container = obj.parent
+            if not 'Selection' in pyatspi.listInterfaces(container):
+                return []
+
         result = []
         acss = self.voice(SYSTEM)
-        # TODO - JD: Is there a better way to do this other than
-        # hard-coding it in?
-        #
-        if args.get('role', obj.getRole()) \
-                in [pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS]:
-            obj = obj.parent
-        childCount = obj.childCount
-        selectedItems = []
-        totalSelectedItems = 0
-        currentItem = 0
-        for child in obj:
-            state = child.getState()
-            if state.contains(pyatspi.STATE_SELECTED):
-                totalSelectedItems += 1
-                selectedItems.append(child)
-            if state.contains(pyatspi.STATE_FOCUSED):
-                currentItem = child.getIndexInParent() + 1
-        countString = messages.selectedItemsCount(totalSelectedItems, childCount)
-        result.append(countString)
+        childCount = container.childCount
+        selectedCount = len(self._script.utilities.selectedChildren(container))
+        result.append(messages.selectedItemsCount(selectedCount, childCount))
         result.extend(acss)
         result.append(self._script.formatting.getString(
                           mode='speech',
                           stringType='iconindex') \
-                      % {"index" : currentItem,
+                      % {"index" : obj.getIndexInParent() + 1,
                          "total" : childCount})
         result.extend(acss)
         return result
@@ -1431,16 +1483,18 @@ class SpeechGenerator(generator.Generator):
         specifications) containing the names of all the selected items.
         This object will be an icon panel or a layered pane.
         """
-        result = []
-        # TODO - JD: Is there a better way to do this other than
-        # hard-coding it in?
-        #
-        if args.get('role', obj.getRole()) == pyatspi.ROLE_ICON:
-            obj = obj.parent
-        for child in obj:
-            if child.getState().contains(pyatspi.STATE_SELECTED):
-                result.extend(self._generateLabelAndName(child))
-        return result
+
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        container = obj
+        if not 'Selection' in pyatspi.listInterfaces(container):
+            container = obj.parent
+            if not 'Selection' in pyatspi.listInterfaces(container):
+                return []
+
+        selectedItems = self._script.utilities.selectedChildren(container)
+        return list(map(self._generateLabelAndName, selectedItems))
 
     def _generateUnfocusedDialogCount(self, obj,  **args):
         """Returns an array of strings (and possibly voice and audio
@@ -1480,50 +1534,39 @@ class SpeechGenerator(generator.Generator):
         previous object with focus.
         """
         result = []
-        acss = self.voice(DEFAULT)
         priorObj = args.get('priorObj', None)
-        requireText = args.get('requireText', True)
         commonAncestor = self._script.utilities.commonAncestor(priorObj, obj)
+        try:
+            role = commonAncestor.getRole()
+        except:
+            pass
+        else:
+            if role == pyatspi.ROLE_COMBO_BOX:
+                return []
+
+        skipRoles = args.get('skipRoles', [])
+        stopAtRoles = args.get('stopAtRoles', [])
+        stopAtRoles.append(pyatspi.ROLE_APPLICATION)
         if obj != commonAncestor:
             parent = obj.parent
-            if parent \
-                and (obj.getRole() == pyatspi.ROLE_TABLE_CELL) \
-                and (parent.getRole() == pyatspi.ROLE_TABLE_CELL):
-                parent = parent.parent
-            while parent and (parent.parent != parent):
-                if parent == commonAncestor:
+            while parent and not parent in [commonAncestor, parent.parent]:
+                parentRole = parent.getRole()
+                if parentRole in stopAtRoles:
                     break
-                if not self._script.utilities.isLayoutOnly(parent):
-                    text = self._script.utilities.displayedLabel(parent)
-                    if not text \
-                       and (not requireText \
-                            or (requireText \
-                                and 'Text' in pyatspi.listInterfaces(parent))):
-                        text = self._script.utilities.displayedText(parent)
-                    try:
-                        pRole = parent.getRole()
-                    except:
-                        pRole = None
-                    if not text \
-                       and pRole in [pyatspi.ROLE_MENU, pyatspi.ROLE_PAGE_TAB]:
-                        text = parent.name
-                    if text and len(text.strip()):
-                        roleInfo = self._generateRoleName(parent)
-                        if roleInfo:
-                            roleInfo.reverse()
-                        # Push announcement of cell to the end
-                        #
-                        if pRole not in \
-                               [pyatspi.ROLE_TABLE_CELL, pyatspi.ROLE_FILLER]:
-                            result.extend(roleInfo)
-                        result.extend(acss)
-                        result.append(text)
-                        if pRole == pyatspi.ROLE_TABLE_CELL:
-                            result.extend(roleInfo)
+                if parentRole not in skipRoles \
+                   and not self._script.utilities.isLayoutOnly(parent):
+                    result.append(self.generate(parent, formatType='focused'))
                 parent = parent.parent
         result.reverse()
         return result
 
+    def _generateOldAncestors(self, obj, **args):
+        """Returns an array of strings (and possibly voice and audio
+        specifications) that represent the text of the ancestors for
+        the object being left."""
+
+        return []
+
     def _generateNewAncestors(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) that represent the text of the ancestors for
@@ -1537,7 +1580,14 @@ class SpeechGenerator(generator.Generator):
         with focus.
         """
         result = []
-        if args.get('priorObj', None):
+        priorObj = args.get('priorObj')
+        if obj.getRole() == pyatspi.ROLE_MENU_ITEM \
+           and (not priorObj or priorObj.getRole() == pyatspi.ROLE_WINDOW):
+            return result
+
+        topLevelObj = self._script.utilities.topLevelObject(obj)
+        if priorObj \
+           or (topLevelObj and topLevelObj.getRole() == pyatspi.ROLE_DIALOG):
             result = self._generateAncestors(obj, **args)
         return result
 
@@ -1864,7 +1914,7 @@ class SpeechGenerator(generator.Generator):
         voicename = voiceType.get(key) or voiceType.get(DEFAULT)
         voices = _settingsManager.getSetting('voices')
         rv = voices.get(voicename)
-        if rv and rv.get('established') == False:
-            rv.pop('established')
+        if not rv or rv.get('established') == False:
+            rv = voices.get(voiceType.get(DEFAULT))
 
-        return [rv]
+        return [acss.ACSS(rv)]
